/***************************************************************************
 *   Copyright (C) 2007 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This file is part of the papyrus library.                             *
 *                                                                         *
 *   papyrus is free software; you can redistribute it and/or modify       *
 *   it under the terms of the GNU Lesser General Public License           *
 *   version 3.0 as published by the Free Software Foundation.             *
 *                                                                         *
 *   papyrus is distributed in the hope that it will be useful,            *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU Lesser General Public License version 3.0 for more details.       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with the papyrus library. If not, see                   *
 *   <http://www.gnu.org/licenses/>.                                       *
 ***************************************************************************/
#include "reference.h"

#include <papyrus/group.h>

namespace Papyrus
{

  Reference::Reference ( const Glib::ustring& id, Drawable::pointer ref ) :
      Drawable ( id ),
      m_referenced ( ref )
  {}

  Reference::pointer Reference::create ( Drawable::pointer ref )
  {
    PAPYRUS_CREATE ( Reference ( Glib::ustring(), ref ) );
  }

  Reference::pointer Reference::create ( const Glib::ustring& id, Drawable::pointer ref )
  {
    PAPYRUS_CREATE ( Reference ( id, ref ) );
  }

  Reference::~Reference()
  {}

  void Reference::draw ( Cairo::RefPtr< Cairo::Context > cairo ) const
  {
    if ( m_referenced )
      m_referenced->draw ( cairo );
  }

  Drawable::pointer Reference::referenced()
  {
    return m_referenced;
  }

  void Reference::set_referenced ( Drawable::pointer d )
  {
    m_referenced = d;
    this->redraw();
  }

  Region Reference::extents(const Matrix& m, ExtentsPerformance ep) const  {
    if ( m_referenced ) return m_referenced->extents(m, ep);
    return Drawable::extents(m, ep);
  }

  Matrix Reference::global_matrix() const
  {
    Matrix local_matrix;
    if ( m_referenced )
      local_matrix = this->composed_matrix() * m_referenced->composed_matrix();
    else
      local_matrix = this->composed_matrix();

    if ( m_parent )
      return m_parent->global_matrix() * local_matrix;
    else
      return local_matrix;
  }

  Glib::ustring Reference::svg(unsigned depth)
  {
    Glib::ustring s;

    s += svg_spacing(depth) + "<use " + svg_id();

    if ( m_referenced )
      s += "xlink:href=\"#" + m_referenced->id() + "\" ";

    s += svg_transform();

    s += " />\n";
    
    return s;
  }

}

